@@ -2,8 +2,9 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2022 Osimis S.A., Belgium
- * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
+ * Copyright (C) 2017-2023 Osimis S.A., Belgium
+ * Copyright (C) 2024-2024 Orthanc Team SRL, Belgium
+ * Copyright (C) 2021-2024 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -45,10 +46,10 @@ namespace Orthanc
     std::string seriesUid_;
     std::string instanceUid_;
 
-    std::string patientHash_;
-    std::string studyHash_;
-    std::string seriesHash_;
-    std::string instanceHash_;
+    mutable std::string patientHash_;
+    mutable std::string studyHash_;
+    mutable std::string seriesHash_;
+    mutable std::string instanceHash_;
 
     void Setup(const std::string& patientId,
                const std::string& studyUid,
@@ -71,12 +72,12 @@ namespace Orthanc
 
     const std::string& GetInstanceUid() const;
 
-    const std::string& HashPatient();
+    const std::string& HashPatient() const;
 
-    const std::string& HashStudy();
+    const std::string& HashStudy() const;
 
-    const std::string& HashSeries();
+    const std::string& HashSeries() const;
 
-    const std::string& HashInstance();
+    const std::string& HashInstance() const;
   };
 }
