@@ -44,7 +44,7 @@
 namespace Orthanc
 {
   class OrthancPluginDatabase::Transaction :
-    public BaseDatabaseWrapper::BaseTransaction,
+    public BaseCompatibilityTransaction,
     public Compatibility::ICreateInstance,
     public Compatibility::IGetChildrenMetadata,
     public Compatibility::ILookupResources,
@@ -564,7 +564,7 @@ namespace Orthanc
     
     virtual void ApplyLookupResources(std::list<std::string>& resourcesId,
                                       std::list<std::string>* instancesId,
-                                      const std::vector<DatabaseConstraint>& lookup,
+                                      const DatabaseDicomTagConstraints& lookup,
                                       ResourceType queryLevel,
                                       const std::set<std::string>& labels,
                                       LabelsConstraint labelsConstraint,
@@ -586,20 +586,20 @@ namespace Orthanc
         std::vector<OrthancPluginDatabaseConstraint> constraints;
         std::vector< std::vector<const char*> > constraintsValues;
 
-        constraints.resize(lookup.size());
-        constraintsValues.resize(lookup.size());
+        constraints.resize(lookup.GetSize());
+        constraintsValues.resize(lookup.GetSize());
 
-        for (size_t i = 0; i < lookup.size(); i++)
+        for (size_t i = 0; i < lookup.GetSize(); i++)
         {
-          lookup[i].EncodeForPlugins(constraints[i], constraintsValues[i]);
+          lookup.GetConstraint(i).EncodeForPlugins(constraints[i], constraintsValues[i]);
         }
 
         ResetAnswers();
         answerMatchingResources_ = &resourcesId;
         answerMatchingInstances_ = instancesId;
       
-        CheckSuccess(that_.extensions_.lookupResources(that_.GetContext(), that_.payload_, lookup.size(),
-                                                       (lookup.empty() ? NULL : &constraints[0]),
+        CheckSuccess(that_.extensions_.lookupResources(that_.GetContext(), that_.payload_, lookup.GetSize(),
+                                                       (lookup.IsEmpty() ? NULL : &constraints[0]),
                                                        Plugins::Convert(queryLevel),
                                                        limit, (instancesId == NULL ? 0 : 1)));
       }
@@ -806,10 +806,10 @@ namespace Orthanc
     }
 
 
-    virtual void GetAllPublicIds(std::list<std::string>& target,
-                                 ResourceType resourceType,
-                                 int64_t since,
-                                 uint32_t limit) ORTHANC_OVERRIDE
+    virtual void GetAllPublicIdsCompatibility(std::list<std::string>& target,
+                                              ResourceType resourceType,
+                                              int64_t since,
+                                              uint32_t limit) ORTHANC_OVERRIDE
     {
       if (that_.extensions_.getAllPublicIdsWithLimit != NULL)
       {
@@ -1660,4 +1660,9 @@ namespace Orthanc
       LOG(WARNING) << "Received an answer from the database index plugin, but not transaction is active";
     }
   }
+
+  uint64_t OrthancPluginDatabase::MeasureLatency()
+  {
+    throw OrthancException(ErrorCode_NotImplemented);
+  }
 }
