@@ -1157,6 +1157,26 @@ namespace Orthanc
         {
           warning = Warnings_002_InconsistentDicomTagsInDb;
         }
+        else if (name == "W003_DecoderFailure")
+        {
+          warning = Warnings_003_DecoderFailure;
+        }
+        else if (name == "W004_NoMainDicomTagsSignature")
+        {
+          warning = Warnings_004_NoMainDicomTagsSignature;
+        }
+        else if (name == "W005_RequestingTagFromLowerResourceLevel")
+        {
+          warning = Warnings_005_RequestingTagFromLowerResourceLevel;
+        }
+        else if (name == "W006_RequestingTagFromMetaHeader")
+        {
+          warning = Warnings_006_RequestingTagFromMetaHeader;
+        }
+        else if (name == "W007_MissingRequestedTagsNotReadFromDisk")
+        {
+          warning = Warnings_007_MissingRequestedTagsNotReadFromDisk;
+        }
         else
         {
           throw OrthancException(ErrorCode_BadFileFormat, name + " is not recognized as a valid warning name");
