@@ -369,14 +369,4 @@ namespace Orthanc
 
     return clone.release();
   }
-
-
-  void DatabaseLookup::AddLabel(const std::string& label)
-  {
-    if (!label.empty())
-    {
-      ServerToolbox::CheckValidLabel(label);
-      labels_.insert(label);
-    }
-  }
 }
